#include <stdio.h>

/*
 * this is an ishft function coded in C for f2c
 * (not needed for g77)
 *
 * v1.0  w. ebisuzaki
 */

unsigned int ishft_(unsigned int *i, int *shift) {

	if ((*shift) > 0) return ((*i) << (*shift));
	if ((*shift) < 0) return ((*i) >> -(*shift));
	return (*i);
}
