C-----------------------------------------------------------------------
      SUBROUTINE BAOPENR(LU,CFN,IRET)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM: BAOPENR        BYTE-ADDRESSABLE OPEN FOR READ-ONLY
C   PRGMMR: IREDELL          ORG: W/NMC23     DATE: 94-04-01
C
C ABSTRACT: OPEN A FILE TO BE ACCESSED BY BAREAD ONLY
C
C PROGRAM HISTORY LOG:
C   95-10-31  IREDELL     WORKSTATION VERSION
C
C USAGE:    CALL BAOPENR(LU,CFN,IRET)
C   INPUT ARGUMENTS:
C     LU           INTEGER UNIT TO OPEN
C     CFN          CHARACTER (*) FILE NAME TO OPEN
C   OUTPUT ARGUMENTS:
C     IRET         INTEGER RETURN CODE (0 IF SUCCESSFUL)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE: WORKSTATIONS
C
C$$$
      CHARACTER CFN*(*)
      CHARACTER CFORM*9
      PARAMETER(CFORM='FORMATTED')
      OPEN(LU,FILE=CFN,IOSTAT=IRET,FORM=CFORM,STATUS='OLD')
      END
