       SUBROUTINE XMOVEX(OUT,IN,IBYTES)
C
C      THIS SUBROUTINE MAY NOT BE NEEDED, ITS WAS IN
C      ASSEMBLER LANGUAGE TO MOVE DATA, IT RAN ABOUT THREE
C      TIMES FASTER THAN A FORTAN DO LOOP, IT WAS USED TO
C      MAKE SURE THE DATA TO BE UNPACKED WAS ON A WORD BOUNDARY,
C      THIS MAY NOT BE NEEDED ON SOME BRANDS OF COMPUTERS.
C
       CHARACTER*1 OUT(*)
       CHARACTER*1 IN(*)
C
       INTEGER     IBYTES
C
       DO 100 I = 1,IBYTES
         OUT(I) = IN(I)
  100  CONTINUE
C
       RETURN
       END
